--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    13:18:09 02/03/06
-- Design Name:    
-- Module Name:    FLASH - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- At power on, this chip automatically starts FPGA configuration from a FLASH memory.
-- After that local bus access is enabled.
--
-- It is possible to program the FPGA directly using slave parellel mode.
-- To do that, first write x"03" to address "10xxxxx", then write x"01" to the same address.
-- read data from address "1xxxxxx" until bit 2 is '1'
-- writing configuration byte data to address "00xxxxx" until all data done
-- 
-- The FLASH memory is programmed in blocks of 256 bytes.
-- To program a FLASH block, first write x"01" to address '0010000" to reset the buffer address counter.
-- Next write 256 bytes of data to address "0010001".
-- FLASH address is set by writing to addresses "1100000"(LSB byte), "1100001"(middle byte) and
-- "1100010"(two MSB bits)
-- To start FLASH programming, First set FLASH address to x"5555", then write x"AA" to address "01xxxxx"
-- Next set FLASH address to x"2AAA", then write x"55" to address "01xxxxx"    
-- Next set FLASH address to x"5555", then write x"A0" to address "01xxxxx"    
-- Finally set FLASH address to the starting address of the block,
-- When FPGA sees that FLASH address has been wriiten, it brings ctrl_in(1) low and starts sending
-- data to the FLASH. After 256 bytes being sent, FPGA brings ctrl_in(1) high again to enable the local bus   
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity test is
    Port ( 
           loc_add : in std_logic_vector(6 downto 0);
           loc_bus : inout std_logic_vector(7 downto 0);
           loc_rw : in std_logic;
           loc_cs : in std_logic;
           maddr : out std_logic_vector(17 downto 0);
           mdata : inout std_logic_vector(7 downto 0);
           moe_b : out std_logic;
           mce_b : out std_logic;
           mwe_b : out std_logic;
           cclk_fb : in std_logic;
           cclk_fbp : inout std_logic;
           cclk : inout std_logic;
			  prog_b : out std_logic
			  );
end test;

architecture Behavioral of test is
signal cntr : std_logic_vector(3 downto 0) := "0000";
signal cntr_b : std_logic_vector(7 downto 0) := "00000000";
signal r_data : std_logic_vector(7 downto 0);
signal w_data : std_logic_vector(7 downto 0);
signal wr_data : std_logic := '0';
signal cmd : std_logic := '0';
signal ce : std_logic := '0';
begin
prog_b <= '0';
cclk_fbp <= cclk_fb;
cclk <= not cclk_fbp;
maddr <= "0000000000" & cntr_b when wr_data = '1' else
         "00" & x"5555" when cntr(2) = '1' else
         "00" & x"2aaa" when cntr(3) = '1' else
			"00000000000" & loc_add;
w_data <= '1' & cntr_b(6 downto 0) when wr_data = '1' else
		  x"aa" when cntr(3 downto 2) = "01" else
        x"55" when cntr(3 downto 2) = "10" else
        x"a0";
process(cclk)
begin
  if(cclk'event and cclk = '1')then
    if(cmd = '0')then
	   cntr <= "0000";
		wr_data <= '0';
	 else
	   cntr <= cntr + 1;
		if(cntr(3 downto 0) = x"f")then
		  wr_data <= '1';
		end if;
	 end if;
    if(wr_data = '0')then
	   cntr_b <= "00000000";
	 elsif(cntr(1 downto 0) = "11")then
	   cntr_b <= cntr_b + 1;
	 end if;
	 ce <= (cntr(3) or cntr(2) or (wr_data and cmd)) and not cntr(1);
  end if;
end process;
process(cclk,loc_cs,loc_rw)
begin
  if(loc_cs = '0' and loc_rw = '0' and loc_add = "1010101")then
    cmd <= '1';
  elsif(cclk'event and cclk = '1')then
    if(cntr_b = x"7f" and cntr(1 downto 0) = "11")then
	   cmd <= '0';
	 end if;
  end if;
end process;
loc_bus <= mdata when loc_cs = '0' and loc_rw = '1' else (others => 'Z');
mdata <= w_data when wr_data = '1' or cntr(3 downto 2) /= "00" else (others => 'Z');
moe_b <= cntr(3) or cntr(2) or wr_data;
mce_b <= (loc_cs or cmd) and not ce;
mwe_b <=	not(cntr(3) or cntr(2) or wr_data);
end Behavioral;
